//! Extensions
use std::{
    fmt::{
	Write as _,
    },
};

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub struct HexSlice<'a>(&'a [u8]);

impl<'a> HexSlice<'a>
{
    /// Create a new `HexSlice` wrapper
    #[inline] pub fn new<T>(from: &'a T) -> Self
    where T: AsRef<[u8]> + ?Sized + 'a
    {
	Self(from.as_ref())
    }

    /// Create a hex string output
    pub fn to_hex_string(&self) -> String
    {
	
	let mut output = String::with_capacity(self.0.len()*2);

	write!(output, "{}", self).unwrap();

	output
    }
}

impl std::fmt::Display for HexSlice<'_>
{
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result
    {
	for byte in self.0.iter()
	{
	    write!(f, "{:02x}", byte)?;
	}
	Ok(())
    }
}


pub trait HexSliceExt
{
    fn as_hex(&self) -> HexSlice<'_>;
}

impl<T> HexSliceExt for T
where T: AsRef<[u8]>
{
    #[inline] fn as_hex(&self) -> HexSlice<'_>
    {
	HexSlice(self.as_ref())
    }
}

impl AsRef<[u8]> for HexSlice<'_>
{
    fn as_ref(&self) -> &[u8]
    {
	self.0
    }
}

#[cfg(test)]
mod tests
{
    use super::*;
    #[test]
    fn hex_encode_bytes()
    {
	let input = b"hello world";
	let output = input.as_hex();
	assert_eq!("68656c6c6f20776f726c64", output.to_hex_string());
	assert_eq!(input.as_hex(), output.as_hex());
	assert_eq!(format!("{}", output), "68656c6c6f20776f726c64");
    }
}
